/* 
**
** A simple screensaver, displays the text "BeOS" at random locations.
**
** Version: 2.0
**
** 
** Copyright (c) 2002 Marcus Overhagen, 2004 Jim Saxton ... FAT ELK SOFTWARE 

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#include <ScreenSaver.h>
#include <View.h>
#include <StringView.h>
#include <Font.h>
#include <stdlib.h>

class ScreenSaver : public BScreenSaver
{
public:
	ScreenSaver(BMessage *archive, image_id);
	void Draw(BView *view, int32 frame);
	void StartConfig(BView *view);
	status_t StartSaver(BView *view, bool preview);
	
private:
	const char *fText;
	int red;
	int green;
	int blue;
	int colors;
	float fX;
	float fY;
	float fSizeX;
	float fSizeY;
	float fTextHeight;
	float fTextWith;
	bool fIsPreview;
};

BScreenSaver *instantiate_screen_saver(BMessage *msg, image_id image) 
{ 
	return new ScreenSaver(msg, image);
} 

ScreenSaver::ScreenSaver(BMessage *archive, image_id id) :
	BScreenSaver(archive, id),
	fText("BeOS"),
	fX(0),
	fY(0)
{
}

void 
ScreenSaver::StartConfig(BView *view) 
{ 
	view->AddChild(new BStringView(BRect(20,10,250,35), "", "BeOS Screensaver, by Marcus Overhagen"));
	view->AddChild(new BStringView(BRect(20,40,200,65), "", "modified by Jim Saxton."));
} 

status_t 
ScreenSaver::StartSaver(BView *view, bool preview)
{
	// save view dimensions and preview mode
	fIsPreview = preview;
	fSizeX = view->Bounds().Width();
	fSizeY = view->Bounds().Height();
	
	// set a new font, about 1/8th of view height, and bold
	BFont font;
	view->GetFont(&font);
	font.SetSize(fSizeY / 8);
	font.SetFace(B_BOLD_FACE);
	view->SetFont(&font);
	
	// find out space needed for text display
	BRect rect;
	escapement_delta delta;
	delta.nonspace = 0;
	delta.space = 0;
	font.GetBoundingBoxesForStrings(&fText,1,B_SCREEN_METRIC,&delta,&rect);
	fTextHeight = rect.Height();
	fTextWith = rect.Width();
	
	// seed the random number generator
	srand((int)system_time());
	
	return B_OK;
}

void 
ScreenSaver::Draw(BView *view, int32 frame)
{
	if (frame == 0) { 
		// fill with black on first frame
		view->SetLowColor(0, 0, 0); 
		view->FillRect(view->Bounds(), B_SOLID_LOW); 
	} else {
		// erase old text on all other frames
		view->SetHighColor(0,0,0);
		view->DrawString(fText,BPoint(fX,fY));
	}

	// find some new text coordinates
	fX = rand() % int(fSizeX - fTextWith);
	fY = rand() % int(fSizeY - fTextHeight - (fIsPreview ? 2 : 20)) + fTextHeight;

	// draw new text
	red = rand() % 255;
	
	blue = rand() % 255;
	
	green = rand () % 255;
	
	colors=red;
	colors=colors+blue;
	colors = colors+green;
	if (colors < 250)
	{
	red=255;
	}
	view->SetHighColor(red,green,blue);
	view->DrawString(fText,BPoint(fX,fY));

	// randomize time until next update (preview mode is faster)
	SetTickSize(((rand() % 4) + 1) * (fIsPreview ? 300000 : 1000000));
}

